unit Unit3;

interface

uses
  Classes, windows, mmsystem, sysutils, syncobjs;

type
  Trxer = class(TThread)
  private
    { Private declarations }
  protected
    procedure Execute; override;
  public
    cs:Tcriticalsection;
    status:string;
    comh:Thandle;
    baudrate,bytesize:integer;
    expected:extended;
    devicename:string;
    perffrq:int64;
    accumerror,accumerror2:double;   //plus naar extern er hurtig
    pipct:longword;
    valid:boolean;
    constructor Create(suspended:boolean);
    destructor Destroy; override;
  end;

implementation

{ Important: Methods and properties of objects in visual components can only be
  used in a method called using Synchronize, for example,

      Synchronize(UpdateCaption);

  and UpdateCaption could look like,

    procedure rxer.UpdateCaption;
    begin
      Form1.Caption := 'Updated in a thread';
    end; }

{ rxer }

constructor Trxer.Create(suspended: boolean);
begin
  cs:=Tcriticalsection.create;
  inherited create(suspended);
end;

destructor Trxer.Destroy;
begin
  inherited;
  cs.free;
end;

procedure Trxer.Execute;
var comh:Thandle;
    DCB: TDCB;
    Config : String;
    CommTimeouts : TCommTimeouts;
    c:byte;
    norcv:integer;
    ts,t0,trel:int64;
    bytesread:longword;
    firstsample:boolean;
    sampleno:int64;
    inputlen,xt,lt,lokaltiklen:extended;
begin
  status:='initializing';
  Comh := CreateFile(pchar(DeviceName), GENERIC_READ or GENERIC_WRITE, 0, Nil,
                                 OPEN_EXISTING,
                                 FILE_ATTRIBUTE_NORMAL, 0);

  if Comh = INVALID_HANDLE_VALUE then begin
    status:='Could not create handle for port';
    exit;
  end;

  if not SetupComm(Comh, {RxBufferSize}32758, {TxBufferSize}8192) then begin
    status:='Could not set comm buffer size';
    CloseHandle(comh);
    exit;
  end;

  if not GetCommState(Comh, DCB) then begin
    status:='Could not get comm state';
    CloseHandle(comh);
    exit;
  end;

  case bytesize of
    5:bytesize:=DATABITS_5;
    6:bytesize:=DATABITS_6;
    7:bytesize:=DATABITS_7;
    8:bytesize:=DATABITS_8;
    16:bytesize:=DATABITS_16;
  end;

  DCB.BaudRate:=baudrate;
  DCB.ByteSize:=bytesize;
  DCB.StopBits:=0;
  dcb.Parity:=NOPARITY;
  dcb.Flags:=  {binary}  1;

  if not SetCommState(Comh, DCB) then begin
    status:='Could not set comm state';
    CloseHandle(comh);
    exit;
  end;

  with CommTimeouts do
  begin
     ReadIntervalTimeout := 0;
     ReadTotalTimeoutMultiplier := 0;
     ReadTotalTimeoutConstant := 500;
     WriteTotalTimeoutMultiplier := 0;
     WriteTotalTimeoutConstant := 500;
  end;

  if not SetCommTimeouts(Comh, CommTimeouts) then  begin
    status:='Could not set comm timeouts';
    CloseHandle(comh);
    exit;
  end;

  setpriorityclass(getcurrentprocess,REALTIME_PRIORITY_CLASS);
  priority:=tpTimeCritical;

  //wait for start-of-packet.

  timebeginperiod(1);
  queryperformancefrequency(perffrq);
  queryperformancecounter(t0);

  norcv:=0;
  t0:=timegettime;
  firstsample:=true;
  sampleno:=0;
  inputlen:=1.0/expected;
  lokaltiklen:=1.0/perffrq;
  lt:=0;
  status:='Running';
  repeat //main nmea loop
      if not ReadFile(Comh, c, 1, BytesRead, Nil) then begin
        status:='Read failure';
        CloseHandle(comh);
        exit;
      end;
      queryperformancecounter(ts);
      trel:=ts-t0;
      if bytesread=0 then begin
        inc(norcv);
{        if norcv=10 then begin
          closehandle(comh);
          status:='no pulses received for 5 seconds';
          exit;
        end;}
      end else begin
        //trel is the timestamp, inputno is the sample.
        sampleno:=sampleno+1;
        inc(pipct);

        if not valid then if trel>perffrq then begin
          valid:=true;
          sampleno:=0;
          t0:=ts;
          trel:=0;
        end;

        xt:=sampleno*inputlen;
        lt:=trel*lokaltiklen;
        cs.enter;
        accumerror:=xt-lt;
        if firstsample then accumerror2:=accumerror;
        accumerror2:=accumerror2*(1-inputlen)+inputlen*accumerror;
        cs.leave;
        if valid then firstsample:=false;
      end;
  until terminated;
  timeendperiod(1);
  closehandle(comh);  { Place thread code here }
  status:='Exited';

end;

end.
